@@ -138,9 +138,13 @@ module Dnsruby
         @@sockets << query_settings.socket
         @@socket_is_persistent[query_settings.socket] = query_settings.is_persistent_socket
       }
+      wake_up_select_thread
+    end
+
+    def wake_up_select_thread
       begin
         @@wakeup_sockets[0].send("wakeup!", 0)
-      rescue Exception => e
+      rescue Exception
         #          do nothing
       end
     end
@@ -190,12 +194,11 @@ module Dnsruby
         end
         #         next if (timeout < 0)
         begin
-          ready, write, errors = IO.select(sockets, nil, nil, timeout)
+          ready, _write, _errors = IO.select(sockets, nil, nil, timeout)
         rescue SelectWakeup
           #  If SelectWakeup, then just restart this loop - the select call will be made with the new data
           next
-        rescue IOError, EncodeError => e
-          #           print "IO Error  =: #{e}\n"
+        rescue IOError, EncodeError
           exceptions = clean_up_closed_sockets
           exceptions.each { |exception| send_exception_to_client(*exception) }
 
@@ -248,7 +251,7 @@ module Dnsruby
     # Removes closed sockets from @@sockets, and returns an array containing 1
     # exception for each closed socket contained in @@socket_hash.
     def clean_up_closed_sockets
-      exceptions = @@mutex.synchronize do
+      @@mutex.synchronize do
         closed_sockets_in_hash = @@sockets.select(&:closed?).select { |s| @@socket_hash[s] }
         @@sockets.delete_if { | socket | socket.closed? }
         closed_sockets_in_hash.each_with_object([]) do |socket, exceptions|
@@ -257,6 +260,7 @@ module Dnsruby
           end
         end
       end
+      exceptions
     end
 
     def process_error(errors)
@@ -295,7 +299,6 @@ module Dnsruby
       @@mutex.synchronize do
         ids = get_active_ids(@@query_hash, msg.header.id)
         return if ids.empty? # should be only one
-        query_settings = @@query_hash[ids[0]].clone
       end
 
       answerip = msg.answerip.downcase
@@ -369,21 +372,25 @@ module Dnsruby
     end
 
     def remove_id(id)
-
       @@mutex.synchronize do
-        socket = @@query_hash[id].socket
-        @@timeouts.delete(id)
-        @@query_hash.delete(id)
-        @@socket_hash[socket].delete(id)
+        remove_id_from_mutex_synchronized_block(id)
+      end
+    end
 
-        decrement_remaining_queries(socket) if persistent?(socket)
+    # THIS MUST BE CALLED FROM INSIDE A @@mutex SYNCHRONISED BLOCK!
+    def remove_id_from_mutex_synchronized_block(id)
+      socket = @@query_hash[id].socket
+      @@timeouts.delete(id)
+      @@query_hash.delete(id)
+      @@socket_hash[socket].delete(id)
 
-        if !persistent?(socket) || max_attained?(socket)
-          @@sockets.delete(socket)
-          @@socket_hash.delete(socket)
-          Dnsruby.log.debug("Closing socket #{socket}")
-          socket.close rescue nil
-        end
+      decrement_remaining_queries(socket) if persistent?(socket)
+
+      if !persistent?(socket) || max_attained?(socket)
+        @@sockets.delete(socket)
+        @@socket_hash.delete(socket)
+        Dnsruby.log.debug("Closing socket #{socket}")
+        socket.close rescue nil
       end
     end
 
@@ -415,12 +422,12 @@ module Dnsruby
       #  Keep buffer for all TCP sockets, and return
       #  to select after reading available data. Once all data has been received,
       #  then process message.
-      buf=""
+      buf = +""
       expected_length = 0
       @@mutex.synchronize {
         buf, expected_length = @@tcp_buffers[socket]
         if (!buf)
-          buf = ""
+          buf = +""
           expected_length = 2
           @@tcp_buffers[socket]=[buf, expected_length]
         end
@@ -444,7 +451,7 @@ module Dnsruby
 
             return false
           end
-          buf << input
+          buf << input if input
         rescue
           #  Oh well - better luck next time!
           return false
@@ -456,7 +463,7 @@ module Dnsruby
           #  We just read the data_length field. Now we need to start reading that many bytes.
           @@mutex.synchronize {
             answersize = buf.unpack('n')[0]
-            @@tcp_buffers[socket] = ["", answersize]
+            @@tcp_buffers[socket] = [+"", answersize]
           }
           return tcp_read(socket)
         else
@@ -547,12 +554,10 @@ module Dnsruby
               query_header_id = @@query_hash[id].query.header.id
               if (query_header_id == e.partial_message.header.id)
                 #  process the response
-                client_queue = nil
-                res = nil
-                query=nil
                 client_queue = @@query_hash[id].client_queue
                 res = @@query_hash[id].single_resolver
                 query = @@query_hash[id].query
+                remove_id_from_mutex_synchronized_block(id)
 
                 #  NOW RESEND OVER TCP!
                 Thread.new {
@@ -642,6 +647,7 @@ module Dnsruby
           do_select
         }
       end
+      wake_up_select_thread
     end
 
     def push_response_to_select(client_id, client_queue, msg, query, res)
@@ -662,6 +668,7 @@ module Dnsruby
           do_select
         }
       end
+      wake_up_select_thread
     end
 
     def push_validation_response_to_select(client_id, client_queue, msg, err, query, res)
@@ -678,6 +685,7 @@ module Dnsruby
           do_select
         }
       end
+      wake_up_select_thread
     end
 
     def send_queued_exceptions
@@ -732,7 +740,7 @@ module Dnsruby
       }
 
       responses.each do |item|
-        client_id, client_queue, msg, err, query, res = item
+        client_id, client_queue, msg, err, _query, _res = item
         #         push_to_client(client_id, client_queue, msg, err)
         client_queue.push([client_id, Resolver::EventType::VALIDATED, msg, err])
         notify_queue_observers(client_queue, client_id)
